function e2c_buildings(original)
{
    switch (original){
        case "Antenna": return "气象天线";
        case "Barn": return "谷仓";
        case "Collector": return "汲能设施";
        case "Forge": return "锻造厂";
        case "Gateway": return "维度裂隙";
        case "Gym": return "健身房";
        case "Hotel": return "宾馆";
        case "House": return "宅院";
        case "Hub": return "中心枢纽";
        case "Hut": return "小屋";
        case "Laboratory": return "实验室";
        case "Mansion": return "豪宅";
        case "Microchip": return "微型芯片";
        case "Nursery": return "托儿所";
        case "Resort": return "度假村";
        case "Shed": return "窝棚";
        case "Smithy": return "铁匠铺";
        case "Tribute": return "贡品";
        case "Warpstation": return "跃迁核心";
        case "Wormhole": return "虫洞";
    }
}

MODULES["buildings"] = {};
MODULES["buildings"].storageMainCutoff = 0.85;
MODULES["buildings"].storageLowlvlCutoff1 = 0.7;
MODULES["buildings"].storageLowlvlCutoff2 = 0.5;

//Helium
var housingList = ['Hut', 'House', 'Mansion', 'Hotel', 'Resort', 'Gateway', 'Collector', 'Warpstation'];

function needGymystic() {
    return game.upgrades['Gymystic'].allowed - game.upgrades['Gymystic'].done > 0;
}

function safeBuyBuilding(building) {
    if (isBuildingInQueue(building))
        return false;
    if (game.buildings[building].locked)
        return false;
    var oldBuy = preBuy2();

  if (bwRewardUnlocked("DecaBuild")) {
        game.global.buyAmt = 10;
    if (!canAffordBuilding(building)) {
        game.global.buyAmt = 2;
	if (!canAffordBuilding(building))
            game.global.buyAmt = 1;
     }
  }
  else if (bwRewardUnlocked("DoubleBuild")) {
        game.global.buyAmt = 2;
  	if (!canAffordBuilding(building)) 
        game.global.buyAmt = 1;
  }        
  else game.global.buyAmt = 1;

  if (!canAffordBuilding(building)) {
      postBuy2(oldBuy);
      return false;
  }

    game.global.firing = false;
	
    if (building == 'Gym' && getPageSetting('GymWall')) {
        game.global.buyAmt = 1;
    }
    if (building == 'Warpstation' && !game.buildings[building].locked && canAffordBuilding(building)) {
        if (game.buildings.Warpstation.owned < 2) {
            game.global.buyAmt = 'Max';
            game.global.maxSplit = 1;
        } else {
            game.global.buyAmt = 1;
        }
        buyBuilding(building, true, true);
        debug('建造' + game.global.buyAmt + '个跃迁核心', "buildings", '*rocket');
        postBuy2(oldBuy);
        return;
    }
    if (building != 'Trap') debug('建造' + e2c_buildings(building), "buildings", '*hammer2');
    if (!game.buildings[building].locked && canAffordBuilding(building)) {
	    buyBuilding(building, true, true);
    }
    postBuy2(oldBuy);
    return true;
}

function buyFoodEfficientHousing() {
    //Init
    var ignoresLimit = getPageSetting('FoodEfficiencyIgnoresMax')
    var unlockedHousing = ["Hut", "House", "Mansion", "Hotel", "Resort"].filter(b => !game.buildings[b].locked);

    //Resets Border Color
    unlockedHousing.forEach(b => document.getElementById(b).style.border = "1px solid #FFFFFF")

    //Checks for Limits
    if (!ignoresLimit) {
        unlockedHousing = unlockedHousing.filter(b => {
            //Filter out buildings that are past the limits
            if (game.buildings[b].owned < getPageSetting('Max' + b) || getPageSetting('Max' + b) < 1)
                return true;

            //But paints their border before removing them
            document.getElementById(b).style.border = "1px solid orange"
            return false
        })
    }

    //Determines Food Efficiency for each housing
    var buildOrder = unlockedHousing.map(b => ({
        'name': b,
        'ratio': getBuildingItemPrice(game.buildings[b], "food", false, 1) / game.buildings[b].increase.by
    }));

    //Grabs the most Food Efficient Housing
    if (buildOrder.length == 0) return;
    bestFoodBuilding = buildOrder.reduce((best, current) => current.ratio < best.ratio ? current : best)

    //If Food Efficiency Ignores Limit is enabled, then it only buy Huts and Houses here
    if (!ignoresLimit || ["Hut", "House"].includes(bestFoodBuilding.name)) {
        document.getElementById(bestFoodBuilding.name).style.border = "1px solid #00CC01";
        safeBuyBuilding(bestFoodBuilding.name);
    }
}

function buyGemEfficientHousing() {
    var gemHousing = ["Mansion", "Hotel", "Resort", "Gateway", "Collector", "Warpstation"];
    var unlockedHousing = [];
    for (var house in gemHousing) {
        if (game.buildings[gemHousing[house]].locked === 0) {
            unlockedHousing.push(gemHousing[house]);
        }
    }
    var obj = {};
    for (var house in unlockedHousing) {
        var building = game.buildings[unlockedHousing[house]];
        var cost = getBuildingItemPrice(building, "gems", false, 1);
        var ratio = cost / building.increase.by;
        obj[unlockedHousing[house]] = ratio;
        document.getElementById(unlockedHousing[house]).style.border = "1px solid #FFFFFF";
    }
    var keysSorted = Object.keys(obj).sort(function (a, b) {
            return obj[a] - obj[b];
        });
    var bestGemBuilding = null;
    for (var best in keysSorted) {
        var max = getPageSetting('Max' + keysSorted[best]);
        if (max === false) max = -1;
        if (game.buildings[keysSorted[best]].owned < max || max == -1 || (getPageSetting('GemEfficiencyIgnoresMax') && keysSorted[best] != "Gateway")) {
            bestGemBuilding = keysSorted[best];
            document.getElementById(bestGemBuilding).style.border = "1px solid #00CC00";

            //Gateway Wall
            if (bestGemBuilding == "Gateway" && getPageSetting('GatewayWall') > 1) {
                if (getBuildingItemPrice(game.buildings.Gateway, "fragments", false, 1) > (game.resources.fragments.owned / getPageSetting('GatewayWall'))) {
                    document.getElementById(bestGemBuilding).style.border = "1px solid orange";
                    bestGemBuilding = null;
                    continue;
                }
            }

            var skipWarp = false;
            if (getPageSetting('WarpstationCap') && bestGemBuilding == "Warpstation") {
                var firstGigaOK = MODULES["upgrades"].autoGigas == false || game.upgrades.Gigastation.done > 0;
                var gigaCapped = game.buildings.Warpstation.owned >= (Math.floor(game.upgrades.Gigastation.done * getPageSetting('DeltaGigastation')) + getPageSetting('FirstGigastation'))
                if (firstGigaOK && gigaCapped) skipWarp = true;
            }
            var warpwallpct = getPageSetting('WarpstationWall3');
            if (warpwallpct > 1 && bestGemBuilding == "Warpstation") {
                if (getBuildingItemPrice(game.buildings.Warpstation, "metal", false, 1) * Math.pow(1 - game.portal.Resourceful.modifier, game.portal.Resourceful.level) > (game.resources.metal.owned / warpwallpct))
                    skipWarp = true;
            }
            if (skipWarp)
                bestGemBuilding = null;
            var getcoord = getPageSetting('WarpstationCoordBuy');
            if (getcoord && skipWarp) {
                var toTip = game.buildings.Warpstation;
                if (canAffordBuilding("Warpstation")) {
                    var howMany = calculateMaxAfford(game.buildings["Warpstation"], true);
                    var needCoord = game.upgrades.Coordination.allowed - game.upgrades.Coordination.done > 0;
                    var coordReplace = (game.portal.Coordinated.level) ? (25 * Math.pow(game.portal.Coordinated.modifier, game.portal.Coordinated.level)).toFixed(3) : 25;
                    if (!canAffordCoordinationTrimps()) {
                        var nextCount = (game.portal.Coordinated.level) ? game.portal.Coordinated.currentSend : game.resources.trimps.maxSoldiers;
                        var amtToGo = ((nextCount * 3) - game.resources.trimps.realMax());
                        var increase = toTip.increase.by;
                        if (game.portal.Carpentry.level && toTip.increase.what == "trimps.max") increase *= Math.pow(1.1, game.portal.Carpentry.level);
                        if (game.portal.Carpentry_II.level && toTip.increase.what == "trimps.max") increase *= (1 + (game.portal.Carpentry_II.modifier * game.portal.Carpentry_II.level));
                        if (amtToGo < increase * howMany)
                            bestGemBuilding = "Warpstation";
                    }
                }
            }
            break;
        }
    }
    if (bestGemBuilding) {
        bestBuilding = bestGemBuilding
        safeBuyBuilding(bestGemBuilding);
    }
}

function buyBuildings() {
    var customVars = MODULES["buildings"];
    var oldBuy = preBuy2();
    var hidebuild = (getPageSetting('BuyBuildingsNew')===0 && getPageSetting('hidebuildings')==true);
    game.global.buyAmt = 1;
    if (!hidebuild) {
        buyFoodEfficientHousing();
        buyGemEfficientHousing();
  	}
    if (!hidebuild && getPageSetting('MaxWormhole') > 0 && game.buildings.Wormhole.owned < getPageSetting('MaxWormhole') && !game.buildings.Wormhole.locked) {
        safeBuyBuilding('Wormhole');
    }

    //Gyms:
    if (!game.buildings.Gym.locked && (getPageSetting('MaxGym') > game.buildings.Gym.owned || getPageSetting('MaxGym') == -1)) {
        var skipGym = false;
	
	    //Dynamic Gyms
        if (getPageSetting('DynamicGyms')) {
	        //Target Zone
	        var targetZone = game.global.world;

            //Enemy stats
            var block = calcOurBlock() / (game.global.brokenPlanet ? 2 : 1);
            var pierce = game.global.brokenPlanet ? (getPierceAmt() * (game.global.formation == 3 ? 2 : 1)) : 0;
            var nextGym = game.upgrades.Gymystic.modifier + Math.max(0, game.upgrades.Gymystic.done-1)/100;
            var currentEnemyDamageOK = block > nextGym * calcSpecificEnemyAttack();
            var zoneEnemyDamageOK = block > calcBadGuyDmg(null, getEnemyMaxAttack(game.global.world, 90, 'Snimp', 1.0), true, true) * (1 - pierce);

            //Challenge stats
            var moreBlockThanHealth = block >= nextGym * calcOurHealth(false);
            var crushedOK = game.global.challengeActive != "Crushed";
            var explosiveOK = game.global.challengeActive != "Daily" || typeof game.global.dailyChallenge.explosive == "undefined";
            var challengeOK = moreBlockThanHealth || crushedOK && explosiveOK;

            //Stop buying Gyms if we already have enough block for our current enemy and also a C99 Snimp
            if (currentEnemyDamageOK && zoneEnemyDamageOK && challengeOK) skipGym = true;
	    }
	
	//Gym Wall
        var gymwallpct = getPageSetting('GymWall');
        if (gymwallpct > 1) {
            if (getBuildingItemPrice(game.buildings.Gym, "wood", false, 1) * Math.pow(1 - game.portal.Resourceful.modifier, game.portal.Resourceful.level)
                > (game.resources.wood.owned / gymwallpct))
                    skipGym = true;
	    }

        //ShieldBlock cost Effectiveness:
        if (game.equipment['Shield'].blockNow) {
            var gymEff = evaluateEquipmentEfficiency('Gym');
            var shieldEff = evaluateEquipmentEfficiency('Shield');
            if ((gymEff.Wall) || (gymEff.Factor <= shieldEff.Factor && !gymEff.Wall))
                skipGym = true;
        }
	
	//Buy Gym
        if (!((game.upgrades['Gymystic'].allowed - game.upgrades['Gymystic'].done) > 0) && !skipGym) safeBuyBuilding('Gym');
    }
    
    //Tributes:
    if (!game.buildings.Tribute.locked && !hidebuild && (getPageSetting('MaxTribute') > game.buildings.Tribute.owned || getPageSetting('MaxTribute') == -1))
        safeBuyBuilding('Tribute');
    
    //Nurseries Init
    var nurseryZoneOk = game.global.world >= getPageSetting('NoNurseriesUntil');
    var maxNurseryOk = getPageSetting('MaxNursery') < 0 || game.buildings.Nursery.owned < getPageSetting('MaxNursery');

    var spireNurseryActive = game.global.challengeActive != "Daily" && (game.global.world > 200 && isActiveSpireAT() || game.global.world <= 200 && getPageSetting('IgnoreSpiresUntil') <= 200);
    var nurseryPreSpire = spireNurseryActive && game.buildings.Nursery.owned < getPageSetting('PreSpireNurseries');

    var dailySpireNurseryActive = game.global.challengeActive == "Daily" && (disActiveSpireAT() || game.global.world <= 200 && getPageSetting('dIgnoreSpiresUntil') <= 200);
    var dailyNurseryPreSpire = dailySpireNurseryActive && game.buildings.Nursery.owned < getPageSetting('dPreSpireNurseries');

    var skipNursery = false;
    var nurserywall = getPageSetting('NurseryWall');
        if (nurserywall > 0) {
	    var nurserywood = (getBuildingItemPrice(game.buildings.Nursery, "wood", false, 1) * Math.pow(1 - game.portal.Resourceful.modifier, game.portal.Resourceful.level));
    	    var nurserygem = (getBuildingItemPrice(game.buildings.Nursery, "gems", false, 1) * Math.pow(1 - game.portal.Resourceful.modifier, game.portal.Resourceful.level));
    	    var nurserymetal = (getBuildingItemPrice(game.buildings.Nursery, "metal", false, 1) * Math.pow(1 - game.portal.Resourceful.modifier, game.portal.Resourceful.level));
            if (nurserywood > (game.resources.wood.owned * (nurserywall / 100))) {
                    skipNursery = true;
	    }
	    else if (nurserygem > (game.resources.gems.owned * (nurserywall / 100))) {
                    skipNursery = true;
	    }
	    else if (nurserymetal > (game.resources.metal.owned * (nurserywall / 100))) {
                    skipNursery = true;
	    }
	}

    //Nurseries
    if (game.buildings.Nursery.locked == 0 && !hidebuild && !skipNursery && (nurseryZoneOk && maxNurseryOk || nurseryPreSpire || dailyNurseryPreSpire)) {
        safeBuyBuilding('Nursery');
    }

    postBuy2(oldBuy);
}

function buyStorage() {
    var customVars = MODULES["buildings"];
    var packMod = 1 + game.portal.Packrat.level * game.portal.Packrat.modifier;
    var Bs = {
        'Barn': 'food',
        'Shed': 'wood',
        'Forge': 'metal'
    };
    for (var B in Bs) {
        var jest = 0;
        var owned = game.resources[Bs[B]].owned;
        var max = game.resources[Bs[B]].max * packMod;
        max = calcHeirloomBonus("Shield", "storageSize", max);
        if (game.global.mapsActive && game.unlocks.imps.Jestimp) {
            jest = simpleSeconds(Bs[B], 45);
            jest = scaleToCurrentMap(jest);
        }
        if ((game.global.world == 1 && owned > max * customVars.storageLowlvlCutoff1) ||
            (game.global.world >= 2 && game.global.world < 10 && owned > max * customVars.storageLowlvlCutoff2) ||
            (owned + jest > max * customVars.storageMainCutoff)) {
            if (canAffordBuilding(B) && game.triggers[B].done) {
                safeBuyBuilding(B);
            }
        }
    }
}

//Radon

var RhousingList = ['Hut', 'House', 'Mansion', 'Hotel', 'Resort', 'Gateway', 'Collector'];

function RsafeBuyBuilding(building) {
    if (isBuildingInQueue(building))
        return false;
    if (game.buildings[building].locked)
        return false;
    var oldBuy = preBuy2();

  if (bwRewardUnlocked("DecaBuild")) {
        game.global.buyAmt = 10;
    if (!canAffordBuilding(building)) {
        game.global.buyAmt = 2;
	if (!canAffordBuilding(building))
            game.global.buyAmt = 1;
     }
  }
  else if (bwRewardUnlocked("DoubleBuild")) {
        game.global.buyAmt = 2;
  	if (!canAffordBuilding(building)) 
        game.global.buyAmt = 1;
  }        
  else game.global.buyAmt = 1;

  if (!canAffordBuilding(building)) {
      postBuy2(oldBuy);
      return false;
  }

    game.global.firing = false;
	
    debug('建造' + e2c_buildings(building), "buildings", '*hammer2');
    if (!game.buildings[building].locked && canAffordBuilding(building)) {
	    buyBuilding(building, true, true);
    }
    postBuy2(oldBuy);
    return true;
}

function RbuyFoodEfficientHousing() {
    var foodHousing = ["Hut", "House", "Mansion", "Hotel", "Resort"];
    var unlockedHousing = [];
    for (var house in foodHousing) {
        if (game.buildings[foodHousing[house]].locked === 0) {
            unlockedHousing.push(foodHousing[house]);
        }
    }
    var buildorder = [];
    if (unlockedHousing.length > 0) {
    for (var house in unlockedHousing) {
        var building = game.buildings[unlockedHousing[house]];
        var cost = getBuildingItemPrice(building, "food", false, 1);
        var ratio = cost / building.increase.by;
        buildorder.push({
            'name': unlockedHousing[house],
            'ratio': ratio
        });
        document.getElementById(unlockedHousing[house]).style.border = "1px solid #FFFFFF";
    }
    buildorder.sort(function (a, b) {
        return a.ratio - b.ratio;
    });
    var bestfoodBuilding = null;
    var bb = buildorder[0];
    var max = getPageSetting('RMax' + bb.name);
    if (game.buildings[bb.name].owned < max || max == -1) {
        bestfoodBuilding = bb.name;
    }
    if (smithylogic(bestfoodBuilding, 'wood', false) && bestfoodBuilding) {
        document.getElementById(bestfoodBuilding).style.border = "1px solid #00CC01";
        RsafeBuyBuilding(bestfoodBuilding);
    }
    }
}

function RbuyGemEfficientHousing() {
    var gemHousing = ["Mansion", "Hotel", "Resort", "Gateway", "Collector"];
    var unlockedHousing = [];
    for (var house in gemHousing) {
        if (game.buildings[gemHousing[house]].locked === 0) {
            unlockedHousing.push(gemHousing[house]);
        }
    }
    var obj = {};
    for (var house in unlockedHousing) {
        var building = game.buildings[unlockedHousing[house]];
        var cost = getBuildingItemPrice(building, "gems", false, 1);
        var ratio = cost / building.increase.by;
        obj[unlockedHousing[house]] = ratio;
        document.getElementById(unlockedHousing[house]).style.border = "1px solid #FFFFFF";
    }
    var keysSorted = Object.keys(obj).sort(function (a, b) {
            return obj[a] - obj[b];
        });
    bestBuilding = null;
    for (var best in keysSorted) {
        var max = getPageSetting('RMax' + keysSorted[best]);
        if (max === false) max = -1;
        if (game.buildings[keysSorted[best]].owned < max || max == -1) {
            bestBuilding = keysSorted[best];
            document.getElementById(bestBuilding).style.border = "1px solid #00CC00";
            break;
        }
    }
    if (smithylogic(bestBuilding, 'gems', false) && bestBuilding) {
        RsafeBuyBuilding(bestBuilding);
    }
}

var smithybought = 0;

function mostEfficientHousing() {

    //Housing
    var HousingTypes = ['Hut', 'House', 'Mansion', 'Hotel', 'Resort', 'Gateway', 'Collector'];

    // Which houses we actually want to check
    var housingTargets = [];
    for (var house of HousingTypes) {
        var maxHousing = (getPageSetting('RMax' + house) === -1 ? Infinity : getPageSetting('RMax' + house));
        if (!game.buildings[house].locked && game.buildings[house].owned < maxHousing) {
            housingTargets.push(house);
        }
    }

    var mostEfficient = {
        name: "",
        time: Infinity
    }

    for (var housing of housingTargets) {

        var worstTime = -Infinity;
        var currentOwned = game.buildings[housing].owned;
        for (var resource in game.buildings[housing].cost) {

            // Get production time for that resource
            var baseCost = game.buildings[housing].cost[resource][0];
            var costScaling = game.buildings[housing].cost[resource][1];
            var avgProduction = getPsString(resource, true);
	    if (avgProduction <= 0) avgProduction = 1;
            var housingBonus = game.buildings.Hut.increase.by;
            if (!game.buildings.Hub.locked) { housingBonus += 500;}

            // Only keep the slowest producer, aka the one that would take the longest to generate resources for
            worstTime = Math.max(baseCost * Math.pow(costScaling, currentOwned - 1) / (avgProduction * housingBonus), worstTime);
            if (resource == 'wood' && !Rhyposhouldwood) worstTime = Infinity;
        }

        if (mostEfficient.time > worstTime) {
            mostEfficient.name = housing;
            mostEfficient.time = worstTime;
        }
    }
    if (mostEfficient.name == "") mostEfficient.name = null;

    return mostEfficient.name;
}

function RbuyStorage(buyFood, buyWood, buyMetal) {

    // Simple map for resources to the names of their storage buildings.
    var Resources = {};
    if (buyFood) {Resources.food = "Barn";}
    if (buyWood) {Resources.wood = "Shed";}
    if (buyMetal) {Resources.metal = "Forge";}

    // Check if we're currently running trimple/atlantrimp
    var isOnTrimple = game.global.currentMapId;
    for (var Map in game.global.mapsOwnedArray) {
        if (Map.id == isOnTrimple) {
            if (Map.name == "Atlantrimp" || Map.name == "Trimple Of Doom"){
                isOnTrimple = true;;
            }
        }
    }

    // Calculate whatever would send us over the current max
    var jestImps = {};
    for (var Res in Resources){

        // Calculate maximum resources for given resource
        var resMax = game.resources[Res].max;
        if (game.global.universe == 1) {
            resMax *= 1 + game.portal.Packrat.level * game.portal.Packrat.modifier;
        } else {
            resMax *= 1 + game.portal.Packrat.radLevel * game.portal.Packrat.modifier;
        }
        resMax = calcHeirloomBonus("Shield", "storageSize", resMax, false);

        // Check if we're in a map
        var curRes = game.resources[Res].owned;
        if (game.global.mapsActive)
        {
            if (isOnTrimple) {
                jestImps[Res] = curRes * 2;
            } else {
                jestImps[Res] = curRes + scaleToCurrentMap(simpleSeconds(Res, 45));
            }
        } else {
            jestImps[Res] = curRes * 1.1;
        }


        if (resMax < jestImps[Res]){
            if (canAffordBuilding(Resources[Res]) && !(game.buildings[Resources[Res]].locked)){
                buyBuilding(Resources[Res], true, true, 1);
            }
        }
    }

}

function RbuyBuildings() {
 
    // Storage
    if (game.global.challengeActive == "Hypothermia" && getPageSetting('Rhypostorage')) {

        var hypofarmzone;
        var hypofarmamount;
        var bonfire = game.challenges.Hypothermia.totalBonfires;
	var wood = game.resources.wood.max;
	var woodmax = wood * (1 + game.portal.Packrat.radLevel * game.portal.Packrat.modifier);
        woodmax = calcHeirloomBonus("Shield", "storageSize", woodmax, false);

        hypofarmzone = getPageSetting('Rhypofarmzone');
        hypofarmamount = getPageSetting('Rhypofarmstack');

        var hypoamountfarmindex = hypofarmzone.indexOf(game.global.world);
        var hypoamountzones = hypofarmamount[hypoamountfarmindex];
            
        var currentprice = (1e10 * Math.pow(100, game.challenges.Hypothermia.totalBonfires));
        var targetprice = (currentprice * Math.pow(100, ((hypoamountzones - bonfire) - 1))) * 1.05;
	targetprice += (targetprice / 1000);

	if (game.global.world > (getPageSetting('Rhypofarmzone')[getPageSetting('Rhypofarmzone').length - 1])) {
	    if (!game.global.autoStorage) {
	        toggleAutoStorage(false);
	    }
	}
	
	else {
	    if (game.global.autoStorage) {
                toggleAutoStorage(false);
	    }

            if (targetprice >= 1e10 && ((woodmax * Math.pow(2, game.buildings.Shed.purchased - game.buildings.Shed.owned)) < targetprice)) {
                buyBuilding('Shed', true, true, 1);
            }
    
            RbuyStorage(true, false, true);
	}
    }
    else {
        if (!game.global.autoStorage) {
            toggleAutoStorage(false);
        }
    }

 
    //Smithy
    if (!game.buildings.Smithy.locked && canAffordBuilding("Smithy", false, false, false, false, 1) && Rhyposhouldwood) {
        // On quest challenge
        if (game.global.challengeActive == 'Quest') {
            if (smithybought > game.global.world) {smithybought = 0;}
 
            if (smithybought < game.global.world && (questcheck() == 7 || (RcalcHDratio() * 10 >= getPageSetting('Rmapcuntoff')))) {
                buyBuilding("Smithy", true, true, 1);
                smithybought = game.global.world;
            }
        } else {
            buyBuilding("Smithy", true, true, 1);
        }
    }
 
    //Microchip
    if (!game.buildings.Microchip.locked && canAffordBuilding('Microchip')) {
        buyBuilding('Microchip', true, true, 1);
    }
 
    //Housing
    var HousingTypes = ['Hut', 'House', 'Mansion', 'Hotel', 'Resort', 'Gateway', 'Collector'];
 
    // Which houses we actually want to check
    var housingTargets = [];
    for (var house in HousingTypes) {
        var maxHousing = (getPageSetting('RMax' + house) === -1 ? Infinity : getPageSetting('RMax' + house));
        if (!game.buildings[HousingTypes[house]].locked && game.buildings[HousingTypes[house]].owned < maxHousing) {
            housingTargets.push(house);
        }
    }
 
    var boughtHousing = false;
 
    do {
 
        boughtHousing = false;
        var housing = mostEfficientHousing();
 
        if (housing != null && canAffordBuilding(housing) && game.buildings[housing].purchased < (getPageSetting('RMax' + housing) === -1 ? Infinity : getPageSetting('RMax' + housing))) {
            buyBuilding(housing, true, true, 1);
            boughtHousing = true;
        }
    } while (boughtHousing)
 
    //Tributes
    if (!game.buildings.Tribute.locked) {
        var buyTributeCount = getMaxAffordable(Math.pow(1.05, game.buildings.Tribute.owned) * 10000, game.resources.food.owned,1.05,true);
        
        if (getPageSetting('RMaxTribute') > game.buildings.Tribute.owned) {
            buyTributeCount = Math.min(buyTributeCount, getPageSetting('RMaxTribute') - game.buildings.Tribute.owned);
        }
 	if (getPageSetting('RMaxTribute') < 0 || (getPageSetting('RMaxTribute') > game.buildings.Tribute.owned)) {
            buyBuilding('Tribute', true, true, buyTributeCount);
	}
    }
	
    //Labs
    if (!game.buildings.Laboratory.locked && getPageSetting('Rnurtureon') == true) {
       if (!isBuildingInQueue('Laboratory') && (getPageSetting('RMaxLabs') < 0 || (getPageSetting('RMaxLabs') > game.buildings.Laboratory.owned))) {
            buyBuilding('Laboratory', true, true, 1);
	}
    }
 
}
